/****************************************************************************
*
*	  Module Filename:  Molecule.cpp
*	  Module Name:      Molecule
*	  Description:
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//--------------  Chemistry Subsystem  --------------------------------------------------------------------------
#include    "Molecule.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Reactor Subsystem  --------------------------------------------------------------------------
#include    "Realised_Molecule_Type.h"
#include    "Tracker.h"
#include    "Tracker_Manager.h"
#include    "Reactor.h"

//--------------------------------------------------------------------------------------------------------------

void  Molecule::Attach_Tracker_List(list<Tracker*>* Tracker_List_Ptr)

//  Attach the specified Tracker List to this Molecule. There cannot be an existing list attached

{
    Assert(List_Of_Attached_Trackers_Ptr == NULL, "Molecule::Attach_Tracker_List - Attempt to attach a Tracker List to a Molecule that already has one");
    List_Of_Attached_Trackers_Ptr = Tracker_List_Ptr;
}
//----------------------------------------------------------------------------------------------------------

const Molecule_Type* Molecule::Get_Molecule_Type_Ptr()


//  Return the Molecule's Molecule_Type

{   
    return(Molecule_Type__Ptr);
}
//---------------------------------------------------------------------------------------------------------------

list<Tracker*>*  Molecule::Get_Tracker_List_Ptr()

//  Return a pointer to the Molecule's list of attached Trackers

{
    return  List_Of_Attached_Trackers_Ptr;
}
//----------------------------------------------------------------------------------------------------------

Molecule::Molecule(Realised_Molecule_Type* R_Mol_Type_Ptr,  double Trackers_Per_Mol) 

//  Constructor - Links to Molecule_Type and Realised_Molecule_Type
//  With specified probability, Setup a Tracker list and attach a Tracker.
//  The Molecule_Type is identified on the Tracker

{
    Molecule_Type* Molecule_Type__Ptr = R_Mol_Type_Ptr->Get_Molecule_Type_Ptr();

    if (Random_Sequence::Get_Ptr(Random_Sequence::Two)->Uniform(double(1.0)) <= Trackers_Per_Mol)
    THEN
        CREATE_OBJECT (list<Tracker*>, List_Of_Attached_Trackers_Ptr = new (list<Tracker*>));

//      Create a Tracker

        Tracker* Tracker_Ptr = Reactor::Get_Reactor__Ptr()->Get_Tracker_Manager_Ptr()->Add_Tracker();

//      The Molecule_Type is identified (as the first item) in the Tracker's Path

//        int Time = Reactor::Get_Reactor__Ptr()->Get_Reactor_Time();
        Tracker_Ptr->Add_MolType_To_Path(Molecule_Type__Ptr);

        List_Of_Attached_Trackers_Ptr->push_back(Tracker_Ptr);
    ELSE
        List_Of_Attached_Trackers_Ptr = NULL;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------




