/****************************************************************************
 *
 *	  Header Filename:  ActReq_Requestor.h
 *	  Module Name:      Requestor for an Action_Request
 *	  Description:      Class used to hold parameters for creation of
 *                       an Action_Request, and to trigger its creation
 *
 *	  Author:           Chris Gordon-Smith
 *
 *     Copyright (C) 2008 Chris Gordon-Smith
 *     Email: See http://www.simsoup.info/
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the
 *     Free Software Foundation; either version 2 of the License, or (at your
 *     option) any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *     See the file Licence.txt for a copy of the GNU General Public License.
 *
 *      Amendment History:-
 *
 *              Version	  Date                        Update Description
 *              =======    ===========                 ==================
 *
 *
 *
 *
 ****************************************************************************/

#ifndef ActReq_Requestor_H
#define ActReq_Requestor_H

class	Action_Request;
class   ActReq_Requestor;
class   Input_Record;

class   Mass_Spec;
class   MolType_Spec;
class   PotEnergy_Spec;
class   ACE_Spec;

class   ActReq_Requestor

{
public:
    ActReq_Requestor();
    virtual ~ActReq_Requestor() {};
    int     Get_Action_Time() const  {return Action_Time_;}
    int     Get_Repeat_Interval() const {return Repeat_Interval_;}
    bool    Proc_Rcd_Semantics(Input_Record*  Record_Ptr, string& Error_Text);
    
    virtual Action_Request*  Make_ActReq() = 0;
    static  bool    Check_RecType_And_Make_Rqstr(Input_Record*  Rec_Ptr,
                                                 ActReq_Requestor*&  Rqstr,
                                                 string&  Error_Text);
protected:
    int     Action_Time_;  // Time at which the Action request should be executed
    int     Repeat_Interval_;
    
    bool    Get_Common_Attributes(Input_Record*  Rec_Ptr, string&  Error_Text);
    virtual bool    Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, 
                                                string& Error_Text) = 0;
    
    ActReq_Requestor(const ActReq_Requestor&);
    ActReq_Requestor& operator= (const ActReq_Requestor&);
};
//-----------------------------------------------------------------------------------------------------------------
class Add_Cat_Trans_ReqRqstr: public ActReq_Requestor {
public:
//        bool Process_Text_Input(list <string> Input_Record);
    
    int     Num_To_Add;
    int     Sub_Net_Id;
    string  Reactant_Spec;
    string  Catalyst_Spec;
    string  Product_Spec;
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Add_Constr_ReqRqstr: public ActReq_Requestor 
{
    public:
        Add_Constr_ReqRqstr();
        Action_Request*  Make_ActReq();
        bool    Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, string& Error_Text);
        ACE_Spec*      Get_ACE_Spec_Pt() const {return ACE_Spec_Pt_;}
        MolType_Spec*  Get_R1_Spec_Pt()  const {return R1_Spec_Pt_;}
        MolType_Spec*  Get_R2_Spec_Pt()  const {return R2_Spec_Pt_;}
        MolType_Spec*  Get_P_Spec_Pt()   const {return P_Spec_Pt_;}
        int            Get_Num_To_Add()  const {return Num_To_Add_;}
    private:
    
        int     Num_To_Add_;
        MolType_Spec*  R1_Spec_Pt_;
        MolType_Spec*  R2_Spec_Pt_;
        MolType_Spec*  P_Spec_Pt_;    
        ACE_Spec*      ACE_Spec_Pt_;

        virtual ~Add_Constr_ReqRqstr();
        Add_Constr_ReqRqstr(const Add_Constr_ReqRqstr&);
        Add_Constr_ReqRqstr& operator= (const Add_Constr_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Add_Fission_ReqRqstr: public ActReq_Requestor {
    public:
        Add_Fission_ReqRqstr();
        Action_Request*  Make_ActReq();
        bool    Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, string& Error_Text);
        ACE_Spec*      Get_ACE_Spec_Pt() const {return ACE_Spec_Pt_;}
        MolType_Spec*  Get_P1_Spec_Pt()  const {return P1_Spec_Pt_;}
        MolType_Spec*  Get_P2_Spec_Pt()  const {return P2_Spec_Pt_;}
        MolType_Spec*  Get_R_Spec_Pt()   const {return R_Spec_Pt_;}
        int            Get_Num_To_Add()  const {return Num_To_Add_;}
    private:
    
        int     Num_To_Add_;
        MolType_Spec*  R_Spec_Pt_;    
        MolType_Spec*  P1_Spec_Pt_;
        MolType_Spec*  P2_Spec_Pt_;
        ACE_Spec*      ACE_Spec_Pt_;

        virtual ~Add_Fission_ReqRqstr();
        Add_Fission_ReqRqstr(const Add_Constr_ReqRqstr&);
        Add_Fission_ReqRqstr& operator= (const Add_Fission_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Add_Maint_MolType_Spec_ReqRqstr: public ActReq_Requestor {
public:
    
    
    int    Num_To_Add;   // For Fixed Inflow
    int    Num_Mols_Fixed;     // For Fixed number of Molecules
    double Tracker_Probability;
    int    Molecule_Type_Index;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Add_MolType_ReqRqstr: public ActReq_Requestor

{
    public:
        Add_MolType_ReqRqstr();
        virtual ~Add_MolType_ReqRqstr();        
        Action_Request*  Make_ActReq();
        bool    Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, 
                                            string& Error_Text);
        Mass_Spec*          Get_Mass_Spec_Pt() const {return Mass_Spec_Pt_;}
        PotEnergy_Spec*    Get_PotEnergy_Spec_Pt() const  {return PotEnergy_Spec_Pt_;}
        int    Get_Num_To_Add() const {return Num_To_Add_;}
    private:
        int     Num_To_Add_;
        Mass_Spec*   Mass_Spec_Pt_;
        PotEnergy_Spec*  PotEnergy_Spec_Pt_;

        Add_MolType_ReqRqstr(const Add_MolType_ReqRqstr&);
        Add_MolType_ReqRqstr& operator= (const Add_MolType_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Add_Molecule_ReqRqstr: public ActReq_Requestor 
{
    public:
        Add_Molecule_ReqRqstr();
        virtual ~Add_Molecule_ReqRqstr();        
        Action_Request*  Make_ActReq();
        bool   Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, 
                                           string& Error_Text);
        MolType_Spec*  Get_MolType_Spec_Pt() const {return MolType_Spec_Pt_;}
        int    Get_Num_To_Add() const {return Num_To_Add_;}
    private:
        int    Num_To_Add_;
        MolType_Spec*  MolType_Spec_Pt_;

        Add_Molecule_ReqRqstr(const Add_Molecule_ReqRqstr&);
        Add_Molecule_ReqRqstr& operator= (const Add_Molecule_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Add_Transfm_ReqRqstr: public ActReq_Requestor {

    public:
        Add_Transfm_ReqRqstr();
        virtual ~Add_Transfm_ReqRqstr();
        Action_Request* Make_ActReq();
        bool            Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, 
                                                    string& Error_Text);

        ACE_Spec*       Get_ACE_Spec_Pt() const {return ACE_Spec_Pt_;}
        MolType_Spec*   Get_R_Spec_Pt()   const {return R_Spec_Pt_;}
        MolType_Spec*   Get_P_Spec_Pt()   const {return P_Spec_Pt_;}
        int             Get_Num_To_Add()  const {return Num_To_Add_;}

    private:    
        int             Num_To_Add_;
        MolType_Spec*   R_Spec_Pt_;
        MolType_Spec*   P_Spec_Pt_;    
        ACE_Spec*       ACE_Spec_Pt_;

        Add_Transfm_ReqRqstr(const Add_Transfm_ReqRqstr&);
        Add_Transfm_ReqRqstr& operator= (const Add_Transfm_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Empty_Reactor_ReqRqstr: public ActReq_Requestor {
public:
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class ChemEvol_ReqRqstr: public ActReq_Requestor {

    public:
        ChemEvol_ReqRqstr();
        virtual ~ChemEvol_ReqRqstr();        
        Action_Request*  Make_ActReq();
        bool    Proc_Rcd_Semantics_Specific(Input_Record*  Record_Ptr, 
                                            string& Error_Text);
    private:

        ChemEvol_ReqRqstr(const ChemEvol_ReqRqstr&);
        ChemEvol_ReqRqstr& operator= (const ChemEvol_ReqRqstr&);
};
//-----------------------------------------------------------------------------------------------------------------

class Remove_Fission_ReqRqstr: public ActReq_Requestor {
public:
    int     Num_To_Remove;
    string  Reactant_Spec;
    string  Product1_Spec;
    string  Product2_Spec;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Remove_Construction_ReqRqstr: public ActReq_Requestor {
public:
    int     Num_To_Remove;
    string  Reactant1_Spec;
    string  Reactant2_Spec;
    string  Product_Spec;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Remove_Maint_MolType_Spec_ReqRqstr: public ActReq_Requestor {
public:
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Remove_Transformation_ReqRqstr: public ActReq_Requestor {
public:
    int     Num_To_Remove;
    string  Reactant_Spec;
    string  Product_Spec;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------


class Set_Reactor_Vol_ReqRqstr: public ActReq_Requestor {
public:
    int Reactor_Volume;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Set_Max_Rand_Rate_Const_Constr_ReqRqstr: public ActReq_Requestor {
public:
    double Max_Rate_Constant;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Set_Max_Rand_Rate_Const_Fission_ReqRqstr: public ActReq_Requestor {
public:
    double Max_Rate_Constant;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Set_Max_Rate_Const_Transformation_ReqRqstr: public ActReq_Requestor {
public:
    double Max_Rate_Constant;
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Set_Rate_Const_Model_ReqRqstr: public ActReq_Requestor {
public:
    string Rate_Const_Model_Spec;
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

class Terminate_Scenario_ReqRqstr: public ActReq_Requestor {
public:
    
//bool Process_Text_Input(list <string> Input_Record);
    
//		Action_Request*  Make_ActReq();
};
//-----------------------------------------------------------------------------------------------------------------

#endif  //ActReq_Requestor
