/****************************************************************************
*
*     Header Filename:  Attribute.h
*     Module Name:      Attribute
*     Description:      Helper classes to support Simulation Attributes
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
#include "globldef.h"
#include "Library_Headers.h"

#include "Kernel_Codes.h"

//--------------  Simulation  Subsystem  -----------------------------------------------------------------------
#include "Attribute.h"
//--------------------------------------------------------------------------------------------------------------

using namespace Attribute;
AttrNames*  AttrNames::Instance_Ptr  = 0;       
//----------------------------------------------------------------------------------------------------------------

ACE_Spec::ACE_Spec(bool Rnd_Fg, double Enrgy):

//  Construct an Activated Complex Energy Spec

    Rnd_Fg_ (Rnd_Fg),
    ACE_ (Enrgy)

{
    TRACE("ACE_Spec::ACE_Spec(bool Rnd_Fg, double Enrgy)");
}
//----------------------------------------------------------------------------------------------------------------

ACE_Spec::ACE_Spec(const ACE_Spec& Existing):

//  Copy construct an Activated Complex Energy Spec

    Rnd_Fg_ (Existing.Rnd_Fg_),
    ACE_ (Existing.ACE_)

{
    TRACE("ACE_Spec::ACE_Spec(const ACE_Spec& Existing");
}
//----------------------------------------------------------------------------------------------------------------

AttrNames::AttrNames()  

//  Setup the Attributes and their user input names

{
    TRACE("AttrNames::AttrNames()");
    using namespace Attribute;
  
    Attributes[ACE_Cd]            = "ACE";
    Attributes[Mass_Cd]           = "Mass";
    Attributes[MolType_Cd]        = "Mol_Type";
    Attributes[Num_To_Add_Cd]     = "Num_To_Add";
    Attributes[Pot_Energy_Cd]     = "Pot_Energy";
    Attributes[Product_Cd]        = "Product";
    Attributes[Product1_Cd]       = "Product1";
    Attributes[Product2_Cd]       = "Product2";
    Attributes[Reactant_Cd]       = "Reactant";
    Attributes[Reactant1_Cd]      = "Reactant1";
    Attributes[Reactant2_Cd]      = "Reactant2";
    Attributes[Repeat_Spec_Cd]    = "Repeat";
    Attributes[Time_Cd]           = "Time";
}
//----------------------------------------------------------------------------------------------------------------

string   AttrNames::Cd_To_Name(Attribute::AttrCode  Attr_Code) 

//  Get the Attribute name (text) for the specified attribite code

{
    TRACE("AttrNames::Cd_To_Name"); 
    using namespace Attribute;
    
    map<AttrCode, string>::const_iterator  Iter;
    Iter = Instance_Ptr->Attributes.find(Attr_Code);

    if (Iter == Instance_Ptr->Attributes.end())
    THEN
        Assert (false, "ActReq_Requestor::Get_Attrib_Name - Invalid Input_Attribute_Type Coce");
    ENDIF
    return  (*Iter).second; 
}
//----------------------------------------------------------------------------------------------------------------

void   AttrNames::Make()

//  Make the singleton object

{
    TRACE("AttrNames::Make");

    Assert (Instance_Ptr == 0, "Attribute::Make - The singleton instance already exists");
    CREATE_OBJECT(AttrNames, Instance_Ptr = new AttrNames);
}
//----------------------------------------------------------------------------------------------------------------

Mass_Spec::Mass_Spec(bool Rand_Flag, int Mass):

//  Initialise a Mass Specifier based on the specified parameters

    Rand_Flag_ (Rand_Flag),
    Mass_ (Mass)
{
    TRACE("Mass_Spec::Mass_Spec(bool Rand_Flag, int Mass)");
}
//----------------------------------------------------------------------------------------------------------------

Mass_Spec::Mass_Spec(const Mass_Spec& Existing):

//  Copy the specified Mass Spec

    Rand_Flag_ (Existing.Rand_Flag_),
    Mass_ (Existing.Mass_)
{
    TRACE("Mass_Spec::Mass_Spec(const Mass_Spec& Existing)");
}
//----------------------------------------------------------------------------------------------------------------
MolType_Spec::MolType_Spec(bool Rand_Flag, int MolType_Id):

//  Setup the MolType_Spec object

    Rand_Flag_ (Rand_Flag),
    MolType_Id_ (MolType_Id)
{
    TRACE("MolType_Spec::MolType_Spec(bool Rand_Flag, int MolType_Id)");
}
//----------------------------------------------------------------------------------------------------------------

MolType_Spec::MolType_Spec(const MolType_Spec& Existing):

//  Copy the existing MolType_Spec object
    
    Rand_Flag_ (Existing.Rand_Flag_),
    MolType_Id_ (Existing.MolType_Id_)
{
    TRACE("MolType_Spec(const MolType_Spec&)");
}
//----------------------------------------------------------------------------------------------------------------

PotEnergy_Spec::PotEnergy_Spec(bool Rand_Flag, double PotEnergy):

    Rand_Flag_ (Rand_Flag),
    PotEnergy_ (PotEnergy)

{
    TRACE("PotEnergy_Spec::PotEnergy_Spec(bool Rand_Flag, double Pot_Energy)");
}
//----------------------------------------------------------------------------------------------------------------

PotEnergy_Spec::PotEnergy_Spec(const PotEnergy_Spec& Existing):

//  Copy the PotEnergy Spec 

    Rand_Flag_ (Existing.Rand_Flag_),
    PotEnergy_ (Existing.PotEnergy_)

{
    TRACE("PotEnergy_Spec::PotEnergy_Spec(const PotEnergy_Spec& Existing)");
}
