/****************************************************************************
*
*	  Header Filename:  Tracker.h
*	  Module Name:      Tracker
*	  Description:      Tracks paths between Molecule_Types taken by material 
*                       as Interactions take place in the Reactor
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Tracker_H
#define Tracker_H

class  Molecule_Type;
class Tracker_Manager;


class   Tracker

{
//===== Relationships with other classes ===================================================================
    public:        
        Molecule_Type*  Get_Current_MolType_Ptr();
    private:
        Tracker_Manager*            Tracker_Manager_Ptr;
        list<Molecule_Type*>        Tracker_Path;
        map<Molecule_Type*,int>     MolType_Occurrences;    // Performance feature. Used to determine quickly
                                                            // whether MolType has been visited
        list<Tracker*>::iterator    Iter_To_Place_In_List;  // Can be used to quickly remove a Tracker pointer   
                                                            // from a list in which it has been put
//==========================================================================================================

    public:
        void Add_MolType_To_Path(Molecule_Type* Molecule_Type_Ptr);
        static void  Destroy(Tracker* Tracker_Ptr);
        void Get_Path_As_String(string& Path_As_String);
        int  Get_Path_Length();// {return Tracker_Path.size();};
        list<Tracker*>::iterator Get_Iter_To_Place_In_List() {return Iter_To_Place_In_List;};
        bool Iter_To_First_MolType();
        bool Iter_To_Next_MolType();
        void Set_Iter_To_Place_In_List(list<Tracker*>::iterator List_Position);
        Tracker(Tracker_Manager* Tracker_Mgr_Ptr);

    private:
        list<Molecule_Type*>::iterator  Tracker_Path_Iterator;
        static long int  Last_Tracker_Id;
        long int  Tracker_Id;
        bool  Check_For_New_Cycle(Molecule_Type* Molecule_Type_Ptr);
       ~Tracker();  // Only the Tracker class can delete Tracker    
};
#endif  //Tracker_H
