/*
*	  Filename:         Form_Main.cpp
*	  Module Name:      Form_Main
*	  Description:      Main Form for SimSoup
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*/
//---------------------------------------------------------------------------
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include  "globldef.h"
#include  "Library_Headers.h"

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//----------------  User_Interface_Presentation API --------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//----------------------------------------------------------------------------------------------------------------

//----------------  Kernel Contrtol API --------------------------------------------------------------------------
#include  "Krnl_Ctrl.h"
#include  "Krnl_Vis.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  User Interface Manager ------------------------------------------------------------------------
#include  "Form_Main.h"
#include  "Network_Painter_Prot.h"
#include  "Form_View_Chemistry.h"
#include  "Form_View_Reactor.h"
#include  "Form_System_Monitor.h"
#include  "Form_Simulation_Statistics.h"
#include  "Form_Help.h"
#include  "Form_Simulation_Control.h"
#include  "Simulation_Display_API.h"
//#include  "User_Actions_API.h"
//---------------------------------------------------------------------------------------------------------------

//==================================================================================================

//  Initialise Static Variables

bool  MainForm::Random_Sequences_Initialised = false;
bool  MainForm::RandSeq_Portability_Checked = false;

//    MainForm*  MainForm::Form_Ptr = 0;
//---------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------------------------
void  MainForm::File_Exit()

//   File- Exit menu option selected.  Exit the simulation

{
    exit(0);
}
//----------------------------------------------------------------------------------------------------------------

void  MainForm::File_Close_Scenario()

//  Close the current scenario

{
    TRACE("MainForm::File_Close_Scenario");
    Krnl_Ctrl::Close_Scenario();
    Form_Simulation_Control::Get_Ptr()->Clear_Log();
    Form_Simulation_Control::Get_Ptr()->List_Current_Scenario();       
    Form_System_Monitor::Get_Ptr()->Display_Object_Statistics(); 
    Form_Simulation_Statistics::Get_Ptr()->Clear();

//  Chemistry and Reactor should now be empty, so just display the emptiness!
    if (TForm__View_Chemistry::Get_Ptr()->Is_On_Active_Page())
    THEN
        TForm__View_Chemistry::Get_Ptr()->Display_Network();
    ENDIF
    TForm__View_Chemistry::Get_Ptr()->List_Nodes();

    if (TForm__View_Reactor::Get_Ptr()->Is_On_Active_Page())
    THEN
        TForm__View_Reactor::Get_Ptr()->Display_Network();
    ENDIF
    TForm__View_Reactor::Get_Ptr()->List_Nodes();
    Random_Sequence::Clear_All();
    Random_Sequences_Initialised = false;  // Force random number sequence to be re-initialised
}
//---------------------------------------------------------------------------

bool  MainForm::File_Load_Scenario()

//  Load a scenario and display

{
    TRACE("MainForm::File_Load_Scenario");
    string  Error_Tx;
    string  Log_Tx1 = string("Loading scenario from file...");
    string  Log_Tx2;

//    File_Close_Scenario();  // Neede if last attempt to load a scenario was not successful


    string PathName;
    bool FileSelect_Fl= GUI_Presentation_API::Choose_File(PathName);

    bool Success_Fl;
    if (FileSelect_Fl)  // Check whether a file was selected
    THEN
        Form_Simulation_Control::Get_Ptr()->Clear_Log();
        Form_Simulation_Control::Get_Ptr()->Output_Simple_Log_Entry(Log_Tx1);
        if (not Krnl_Ctrl::Load_Scenario_Series(PathName, Error_Tx))
        THEN
            Log_Tx2 =  Get_Error_Tx()
                    + string(" ") + Error_Tx;
            Success_Fl = false;
        ELSE
            Log_Tx2 = string("Scenario loaded OK.");
            Form_Simulation_Control::Get_Ptr()->List_Current_Scenario(); 
            Success_Fl = true;
        ENDIF
        Form_Simulation_Control::Get_Ptr()->Output_Simple_Log_Entry(Log_Tx2);
        Form_System_Monitor::Get_Ptr()->Display_Object_Statistics();   
    ELSE
        Success_Fl = false;
    ENDIF
    
    return Success_Fl;
}
//----------------------------------------------------------------------------------------------------------------

void    MainForm::Make()

//  Initialise the User Interface Manager
{
    Form_System_Monitor::Make();
    Form_Simulation_Control::Make();
    TForm__View_Chemistry::Make();
    TForm__View_Reactor::Make();
    Form_Help::Make();
    Form_Simulation_Statistics::Make();  

    Random_Sequences_Initialised = false;
}
//-----------------------------------------------------------------------------------------------------------

bool MainForm::Simulation_Go_Click()

//  Run the simulation 

{
    TRACE("MainForm::Simulation_Go_Click");
//  Don't attempt to run if there is no scenario
    if (not Krnl_Vis::Scenario_::Get_Curr_Scenario_Ptr())
    THEN
        Form_Simulation_Control::Get_Ptr()->Output_Log_Entry(
            (string("User attempt to start simulation, but no Scenario is defined")));
        Krnl_Ctrl::Disable_Simulation_Processing();
        return false;  // simulation not started
    ENDIF

//  Initialise Random Number Sequences. Doing it sooner leads to behaviour in 
//  which the pseudo-random number sequence is not reproducible from one run
//  to the next. This is possibly due to interface by UI components, particularly
//  while migrating the code from Lazarus to GTK

    if (Random_Sequences_Initialised == false)
    THEN
        Random_Sequence::Make(Random_Sequence::Num_Sequences);
        Random_Sequence::Get_Ptr(Random_Sequence::One)->Set_Seed(1);
        Random_Sequences_Initialised = true;
    ENDIF

    if (!RandSeq_Portability_Checked)
    THEN
        bool Rand_Sequence_Portable = Random_Sequence::Check_Portability();
    
        if (!Rand_Sequence_Portable)
        THEN 
            Form_Simulation_Control::Get_Ptr()->Output_Simple_Log_Entry(string("**** WARNING **** Random Number Generator Portability Check: The pseudo-random number generator on your system is not producing exactly the same sequence of numbers that is produced in the SimSoup development environment. This is not a serious problem, but it means that the demo scenario will not produce results identical to those shown on the SimSoup website. The results will be equally valid, but based on a different sequence of pseudo-random numbers."));
        ENDIF  
        RandSeq_Portability_Checked = true;
    ENDIF

    Krnl_Ctrl::Enable_Simulation_Processing();

//  While the simulation is running, disable selection on Chem / Reactor ListViews
    GUI_Presentation_API::Set_Selection_Mode(Widget_Ids::MolTypes_ListView,
                                            GUI::None); 
    GUI_Presentation_API::Set_Selection_Mode(Widget_Ids::R_MolTypes_ListView,
                                            GUI::None);  

//  And clear the ListViews
    GUI_Presentation_API::Clear(Widget_Ids::IntTypes_ListStore);
    GUI_Presentation_API::Clear(Widget_Ids::R_IntType_ListStore);
    return true; // Simulation started
}   
//----------------------------------------------------------------------------------------------------------------

void MainForm::Simulation_Stop_Click()

{
    Krnl_Ctrl::Disable_Simulation_Processing();
    Form_System_Monitor::Get_Ptr()->Display_Object_Statistics(); 

//  While the simulation is not running, allow selection on Chem / Reactor ListViews
    GUI_Presentation_API::Set_Selection_Mode(Widget_Ids::MolTypes_ListView,
                                            GUI::Single); 
    GUI_Presentation_API::Set_Selection_Mode(Widget_Ids::R_MolTypes_ListView,
                                            GUI::Single);  
}


