/****************************************************************************
*
*     Header Filename:  Simulation_Display_API.cpp
*     Module Name:      Simulation_Display_API
*     Description:      Interface to User Interface Subsystem for display updates
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

namespace API{
    // Exists only to make KDevelop display Simulation_Display_API in explorer
int Simulation_Display_API;}
#include "globldef.h"
#include "Library_Headers.h"

//--------------  User Interface Manager ------------------------------------------------------------------------

#include "Form_Simulation_Statistics.h"
#include "Simulation_Display_API.h"
#include "Form_Simulation_Control.h"
#include "Form_View_Chemistry.h"
#include "Form_View_Reactor.h"
#include "Form_System_Monitor.h"
//---------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Display_Chemistry()

//  Invoke the User Interface Manager's Chemistry Display functions if necessary

{
    if (TForm__View_Chemistry::Get_Ptr()->Is_On_Active_Page())
    THEN
        TForm__View_Chemistry::Get_Ptr()->Display_Network();
        TForm__View_Chemistry::Get_Ptr()->List_Nodes();
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------


void Simulation_Display_API::Display_Object_Statistics()

//  Invoke the User Interface Manager's Display Object Statistics function if necessary

{
    if (Form_System_Monitor::Get_Ptr()->Is_On_Active_Page())
    THEN
        Form_System_Monitor::Get_Ptr()->Display_Object_Statistics();
    ENDIF
}
//--------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Display_Reactor()

//  Invoke the User Interface Manager's Reactor Display functions if necessary

{
    if (TForm__View_Reactor::Get_Ptr()->Is_On_Active_Page())
    THEN
        TForm__View_Reactor::Get_Ptr()->Display_Network();
        TForm__View_Reactor::Get_Ptr()->List_Nodes();
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Display_Statistics_Optimised(bool Simulation_Running)

//  Invoke the User Interface Manager's Optimised Statistics Display function if necessary

{
    if (Form_Simulation_Statistics::Get_Ptr()->Is_On_Active_Page())
    THEN
        Form_Simulation_Statistics::Get_Ptr()->Display_Statistics(Form_Simulation_Statistics::Optimised, Simulation_Running);
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Output_Log_Entry(string Action_Report)

//  Invoke the User Interface Manager's Output_Log_Entry function

{
    Form_Simulation_Control::Get_Ptr()->Output_Log_Entry(Action_Report);
} 
//---------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Output_Simple_Log_Entry(string Action_Report)

//  Invoke the User Interface Manager's Simple Output_Log_Entry function

{
    Form_Simulation_Control::Get_Ptr()->Output_Simple_Log_Entry(Action_Report);
} 
//---------------------------------------------------------------------------------------------------------------

void Simulation_Display_API::Save_Statistics_Display(string* Filename_Ptr)

//  Save the Simulation Statistics display to a file

{
    Form_Simulation_Statistics::Get_Ptr()->Save_Display_To_File(Filename_Ptr);
}

