/****************************************************************************
*
*	  Header Filename:  Network_Painter_Prot.h
*	  Module Name:      Network_Painter_Prot
*	  Description:      Network_Painter Protocol Class
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Network_Painter_Prot_H
#define Network_Painter_Prot_H


class TBitmap;
enum  Connection_Selection_State   {EOpt_Connection_Selected,EOpt_Connection_Not_Selected};
enum  Enum_Node_Selection_State         {EOpt_Node_Selected_As_Reactant,
                                         EOpt_Node_Selected_As_Catalyst,
                                         EOpt_Node_Selected_As_Product,
                                         EOpt_Node_Not_Selected,
                                         EOpt_Node_Selection_End};

class   Network_Painter_Prot

//  Provides the interface to the Network_Painter class, along with
//  helper class support for the concept of Selection State for Nodes

{

//===== Relationships with other classes ===================================================================
//==========================================================================================================

//  Helper Classes

public:

    class Manage_Node_Selection_State

//  Support functions for use with enumeration type for
//  Node selection states

    {
    public:
        static Enum_Node_Selection_State  First();
        static Enum_Node_Selection_State  Next(Enum_Node_Selection_State Selection_State);
    };

    struct    Screen_Location

//      Used to define a Screen Location in Pixels
    {
        int X;
        int Y;
    };

//  Public Functions

    virtual void Clear() = 0;
                               
    virtual void Draw_Construction(int Reactant1_Node_Index,
                                   int Reactant2_Node_Index,
                                   int Product1_Node_Index,
                                   double Connection_Strength,
                                   int Conn_Selection_State,
                                   bool Bool__Store_Network_Item = true) = 0;
                                   
    virtual void Draw_Fission     (int Reactant1_Node_Index,
                                   int Product1_Node_Index,
                                   int Product2_Node_Index,
                                   double Connection_Strength,
                                   int Conn_Selection_State,
                                   bool Bool__Store_Network_Item = true) = 0;
                                   
    virtual void Draw_Transformation(int Reactant1_Node_Index,
                                    int Product1_Node_Index,
                                    double Connection_Strength,
                                    int Conn_Selection_State,
                                    bool Bool__Store_Network_Item = true) = 0;

    virtual void Draw_Molecule_Type(const int Molecule_Type_Identifier,
                                    const double Node_Strength,
                                    const Enum_Node_Selection_State,
                                    const bool Bool__Store_Network_Item = true) = 0;

    virtual void    Draw_Text_Output(int X_Pos, int Y_Pos,  const char* Output_Text_Ptr,
                                     bool Bool__Store_Network_Item = true) = 0;
    
    virtual ~Network_Painter_Prot() {};
                                
    static Network_Painter_Prot*  Make_Network_Painter(int     NumOfNodesPerDisplayRow,
                                                       int     SpaceInPixelsBetweenNodes,
                                                       double   Typical_Connection_Strength,
                                                       double   Typical_Node_Strength,
                                       //                void*   TPaintBox__NetworkDisplayPtr,
                                                               // Use void* to avoid using VCL here
                                                       bool    Display_All_Connections_Ind,
                                                       Widget_Ids::Drawing_Area_Id Draw_Area_Id);
                                                               
    virtual int Node_Index_For_Screen_Location(const Screen_Location*  const Screen_Location__Ptr) const = 0;
    virtual bool Screen_Location_Is_Valid_Node(const Screen_Location*  const Screen_Location__Ptr,
                                               int  Number_Of_Nodes) const = 0;
    
    virtual void Clear_Network_Bitmap() = 0;
    virtual void Clear_Network_Image() = 0;
    virtual void Display_Network_Image() = 0;
    virtual void Redraw_Network() = 0;
    virtual void Zoom_In() = 0;
    virtual void Zoom_Out() = 0;
    virtual void Increase_Connection_Width() = 0;
    virtual void Decrease_Connection_Width() = 0;
    virtual void Increase_Node_Width() = 0;

    virtual void Decrease_Node_Width() = 0;
    virtual void Control_Unselected_Conn_Display(bool  Flag_Value) = 0;
    virtual void Set_Node_Selection_Colour_Map(Enum_Node_Selection_State Selection_State, 
                                               Colour_Id Selection_Colour) = 0;
    virtual void Set_Number_Of_Nodes(int Num_Nodes) = 0;
                                                                                                 // Use int to avoid using VCL
//    virtual TBitmap* Get_TBitmap_Network_Display_Ptr() const = 0;
};

#endif  //Network_Painter_Prot_H

