/****************************************************************************
*
*     Header Filename:  GUI_Presentation_API.h
*     Module Name:      GUI_Presentation_API
*     Description:      API for the SimSoup Presentation Layer
*                       Isolates application code from the GUI toolkit by avoiding
*                       reference to the toolkit in the interface
*
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version    Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/
#ifndef GUI_PRESENTATION_API_H
#define GUI_PRESENTATION_API_H

#include  "GUI_Item_Ids.h"

class   GUI_Presentation;
class   Indexed_Series;

class   GUI_Presentation_API
{
//===== Relationships with other classes ===================================================================
    private:
        static GUI_Presentation*  GUI_Presentation_Ptr;
//==========================================================================================================

    public:

        static void Start();

//      FileChooser functions

        static bool Choose_File(string&  PathName);

//      Notebook Functions

        static string Get_Active_Page_Label();

//      ListView functions

        static bool Get_Selected_Row(Widget_Ids::ListView_Id Id,
                                     list<string>& List_Of_Col_Texts, 
                                     void*& Data_Ptr);
        static bool Get_Selected_Row(Widget_Ids::ListView_Id Id,
                                     list<string>& List_Of_Col_Texts);
        static void Update_Selected_Row(Widget_Ids::ListView_Id Id, 
                                        list<string>& List_Of_Col_Texts_Ptr, 
                                        void* Data_Ptr);
        static bool Select_Row(Widget_Ids::ListView_Id Id, const string& Row_Tx);

        enum ListView_SelMode {None = 0, Single = 1, Multiple = 2};
        static void Set_Selection_Mode(Widget_Ids::ListView_Id Id,
                                       ListView_SelMode   Mode);

//      ListStore functions

        static void Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts, void* Data_Ptr);
        static void Add_Row(Widget_Ids::ListStore_Id Id, list<string>& List_Of_Col_Texts);
        static void Clear(Widget_Ids::ListStore_Id Id);
        static void Get_Row_N_Col1(Widget_Ids::ListStore_Id Id, int RowNum, string& Row_Tx);


//      Drawing Area functions

        static void Clear(Widget_Ids::Drawing_Area_Id Area_Spec);
        static void Draw_Line(Widget_Ids::Drawing_Area_Id, int X1, int Y1, int X2, int Y2);
        static void Draw_Rectangle(Widget_Ids::Drawing_Area_Id, bool Filled, int X, int Y, int Width, int Height);
        static void Draw_Pixmap(Widget_Ids::Drawing_Area_Id Area_Spec);
        static void Get_Size(Widget_Ids::Drawing_Area_Id   Drawing_Area_Spec, int& Width, int&  Height);
        static void Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id Area_Spec, Colour_Id Colour_Spec);
        static void Set_Foreground_Colour(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, int Red, int Green, int Blue);
        static void Text_Out(Widget_Ids::Drawing_Area_Id Drawing_Area_Spec, int X, int Y, string  Text);

//      TextBuffer Functions

        static void Add_Text(Widget_Ids::TextBuffer_Id Id, string&  Text);
        static void Clear(Widget_Ids::TextBuffer_Id Id);
};

    typedef GUI_Presentation_API GUI;
#endif  //GUI_PRESENTATION_API_H
